@@ -1,5 +1,6 @@
 use crate::auditwheel::{get_policy_and_libs, patchelf, relpath, AuditWheelMode};
 use crate::auditwheel::{PlatformTag, Policy};
+use crate::bridge::Abi3Version;
 use crate::build_options::CargoOptions;
 use crate::compile::{warn_missing_py_init, CompileTarget};
 use crate::module_writer::{
@@ -131,6 +132,8 @@ pub struct BuildContext {
     pub editable: bool,
     /// Cargo build options
     pub cargo_options: CargoOptions,
+    /// Zip compression level
+    pub compression_level: u16,
 }
 
 /// The wheel file location and its Python version tag (e.g. `py3`).
@@ -152,41 +155,78 @@ impl BuildContext {
         let wheels = match self.bridge() {
             BridgeModel::Bin(None) => self.build_bin_wheel(None)?,
             BridgeModel::Bin(Some(..)) => self.build_bin_wheels(&self.interpreter)?,
-            BridgeModel::Bindings { .. } => self.build_binding_wheels(&self.interpreter)?,
-            BridgeModel::BindingsAbi3 { major, minor, .. } => {
-                let abi3_interps: Vec<_> = self
-                    .interpreter
-                    .iter()
-                    .filter(|interp| interp.has_stable_api())
-                    .cloned()
-                    .collect();
-                let non_abi3_interps: Vec<_> = self
-                    .interpreter
-                    .iter()
-                    .filter(|interp| !interp.has_stable_api())
-                    .cloned()
-                    .collect();
-                let mut built_wheels = Vec::new();
-                if !abi3_interps.is_empty() {
-                    built_wheels.extend(self.build_binding_wheel_abi3(
-                        &abi3_interps,
-                        *major,
-                        *minor,
-                    )?);
-                }
-                if !non_abi3_interps.is_empty() {
-                    let interp_names: HashSet<_> = non_abi3_interps
+            BridgeModel::PyO3(crate::PyO3 { abi3, .. }) => match abi3 {
+                Some(Abi3Version::Version(major, minor)) => {
+                    let abi3_interps: Vec<_> = self
+                        .interpreter
                         .iter()
-                        .map(|interp| interp.to_string())
+                        .filter(|interp| interp.has_stable_api())
+                        .cloned()
                         .collect();
-                    eprintln!(
-                        "⚠️ Warning: {} does not yet support abi3 so the build artifacts will be version-specific.",
-                        interp_names.iter().join(", ")
-                    );
-                    built_wheels.extend(self.build_binding_wheels(&non_abi3_interps)?);
+                    let non_abi3_interps: Vec<_> = self
+                        .interpreter
+                        .iter()
+                        .filter(|interp| !interp.has_stable_api())
+                        .cloned()
+                        .collect();
+                    let mut built_wheels = Vec::new();
+                    if !abi3_interps.is_empty() {
+                        built_wheels.extend(self.build_pyo3_wheel_abi3(
+                            &abi3_interps,
+                            *major,
+                            *minor,
+                        )?);
+                    }
+                    if !non_abi3_interps.is_empty() {
+                        let interp_names: HashSet<_> = non_abi3_interps
+                            .iter()
+                            .map(|interp| interp.to_string())
+                            .collect();
+                        eprintln!(
+                                "⚠️ Warning: {} does not yet support abi3 so the build artifacts will be version-specific.",
+                                interp_names.iter().join(", ")
+                            );
+                        built_wheels.extend(self.build_pyo3_wheels(&non_abi3_interps)?);
+                    }
+                    built_wheels
                 }
-                built_wheels
-            }
+                Some(Abi3Version::CurrentPython) => {
+                    let abi3_interps: Vec<_> = self
+                        .interpreter
+                        .iter()
+                        .filter(|interp| interp.has_stable_api())
+                        .cloned()
+                        .collect();
+                    let non_abi3_interps: Vec<_> = self
+                        .interpreter
+                        .iter()
+                        .filter(|interp| !interp.has_stable_api())
+                        .cloned()
+                        .collect();
+                    let mut built_wheels = Vec::new();
+                    if !abi3_interps.is_empty() {
+                        let interp = abi3_interps.first().unwrap();
+                        built_wheels.extend(self.build_pyo3_wheel_abi3(
+                            &abi3_interps,
+                            interp.major as u8,
+                            interp.minor as u8,
+                        )?);
+                    }
+                    if !non_abi3_interps.is_empty() {
+                        let interp_names: HashSet<_> = non_abi3_interps
+                            .iter()
+                            .map(|interp| interp.to_string())
+                            .collect();
+                        eprintln!(
+                                "⚠️ Warning: {} does not yet support abi3 so the build artifacts will be version-specific.",
+                                interp_names.iter().join(", ")
+                            );
+                        built_wheels.extend(self.build_pyo3_wheels(&non_abi3_interps)?);
+                    }
+                    built_wheels
+                }
+                None => self.build_pyo3_wheels(&self.interpreter)?,
+            },
             BridgeModel::Cffi => self.build_cffi_wheel()?,
             BridgeModel::UniFfi => self.build_uniffi_wheel()?,
         };
@@ -419,6 +459,14 @@ impl BuildContext {
         let artifact_dir = match self.bridge() {
             // cffi bindings that contains '.' in the module name will be split into directories
             BridgeModel::Cffi => self.module_name.split(".").collect::<PathBuf>(),
+            // For namespace packages the modules the modules resides resides at ${module_name}.so
+            // where periods are replaced with slashes so for example my.namespace.module would reside
+            // at my/namespace/module.so
+            _ if self.module_name.contains(".") => {
+                let mut path = self.module_name.split(".").collect::<PathBuf>();
+                path.pop();
+                path
+            }
             // For other bindings artifact .so file usually resides at ${module_name}/${module_name}.so
             _ => PathBuf::from(&self.module_name),
         };
@@ -473,7 +521,9 @@ impl BuildContext {
     /// Returns the platform part of the tag for the wheel name
     pub fn get_platform_tag(&self, platform_tags: &[PlatformTag]) -> Result<String> {
         if let Ok(host_platform) = env::var("_PYTHON_HOST_PLATFORM") {
-            return Ok(host_platform.replace(['.', '-'], "_"));
+            let override_platform = host_platform.replace(['.', '-'], "_");
+            eprintln!("🚉 Overriding platform tag from _PYTHON_HOST_PLATFORM environment variable as {override_platform}.");
+            return Ok(override_platform);
         }
 
         let target = &self.target;
@@ -562,7 +612,7 @@ impl BuildContext {
 
                 let mut os = target.target_os().to_string().to_ascii_lowercase();
                 // See https://github.com/python/cpython/blob/46c8d915715aa2bd4d697482aa051fe974d440e1/Lib/sysconfig.py#L722-L730
-                if os.starts_with("sunos") {
+                if target.target_os() == Os::Solaris || target.target_os() == Os::Illumos {
                     // Solaris / Illumos
                     if let Some((major, other)) = release.split_once('_') {
                         let major_ver: u64 = major.parse().context("illumos major version is not a number")?;
@@ -604,7 +654,7 @@ impl BuildContext {
         Ok((tag, tags))
     }
 
-    fn write_binding_wheel_abi3(
+    fn write_pyo3_wheel_abi3(
         &self,
         artifact: BuildArtifact,
         platform_tags: &[PlatformTag],
@@ -621,6 +671,7 @@ impl BuildContext {
             &self.metadata24,
             &[tag.clone()],
             self.excludes(Format::Wheel)?,
+            self.compression_level,
         )?;
         self.add_external_libs(&mut writer, &[&artifact], &[ext_libs])?;
 
@@ -648,7 +699,7 @@ impl BuildContext {
 
     /// For abi3 we only need to build a single wheel and we don't even need a python interpreter
     /// for it
-    pub fn build_binding_wheel_abi3(
+    pub fn build_pyo3_wheel_abi3(
         &self,
         interpreters: &[PythonInterpreter],
         major: u8,
@@ -669,13 +720,8 @@ impl BuildContext {
         } else {
             self.platform_tag.clone()
         };
-        let (wheel_path, tag) = self.write_binding_wheel_abi3(
-            artifact,
-            &platform_tags,
-            external_libs,
-            major,
-            min_minor,
-        )?;
+        let (wheel_path, tag) =
+            self.write_pyo3_wheel_abi3(artifact, &platform_tags, external_libs, major, min_minor)?;
 
         eprintln!(
             "📦 Built wheel for abi3 Python ≥ {}.{} to {}",
@@ -688,7 +734,7 @@ impl BuildContext {
         Ok(wheels)
     }
 
-    fn write_binding_wheel(
+    fn write_pyo3_wheel(
         &self,
         python_interpreter: &PythonInterpreter,
         artifact: BuildArtifact,
@@ -703,6 +749,7 @@ impl BuildContext {
             &self.metadata24,
             &[tag.clone()],
             self.excludes(Format::Wheel)?,
+            self.compression_level,
         )?;
         self.add_external_libs(&mut writer, &[&artifact], &[ext_libs])?;
 
@@ -731,14 +778,14 @@ impl BuildContext {
         ))
     }
 
-    /// Builds wheels for a Cargo project for all given python versions.
+    /// Builds wheels for a pyo3 extension for all given python versions.
     /// Return type is the same as [BuildContext::build_wheels()]
     ///
     /// Defaults to 3.{5, 6, 7, 8, 9} if no python versions are given
     /// and silently ignores all non-existent python versions.
     ///
     /// Runs [auditwheel_rs()] if not deactivated
-    pub fn build_binding_wheels(
+    pub fn build_pyo3_wheels(
         &self,
         interpreters: &[PythonInterpreter],
     ) -> Result<Vec<BuiltWheelMetadata>> {
@@ -755,12 +802,8 @@ impl BuildContext {
             } else {
                 self.platform_tag.clone()
             };
-            let (wheel_path, tag) = self.write_binding_wheel(
-                python_interpreter,
-                artifact,
-                &platform_tags,
-                external_libs,
-            )?;
+            let (wheel_path, tag) =
+                self.write_pyo3_wheel(python_interpreter, artifact, &platform_tags, external_libs)?;
             eprintln!(
                 "📦 Built wheel for {} {}.{}{} to {}",
                 python_interpreter.interpreter_kind,
@@ -806,12 +849,12 @@ impl BuildContext {
             return Ok(artifact);
         }
         // auditwheel repair will edit the file, so we need to copy it to avoid errors in reruns
-        let artifact_path = &artifact.path;
-        let maturin_build = artifact_path.parent().unwrap().join("maturin");
+        let maturin_build = self.target_dir.join(env!("CARGO_PKG_NAME"));
         fs::create_dir_all(&maturin_build)?;
+        let artifact_path = &artifact.path;
         let new_artifact_path = maturin_build.join(artifact_path.file_name().unwrap());
         fs::copy(artifact_path, &new_artifact_path)?;
-        artifact.path = new_artifact_path;
+        artifact.path = new_artifact_path.normalize()?.into_path_buf();
         Ok(artifact)
     }
 
@@ -829,6 +872,7 @@ impl BuildContext {
             &self.metadata24,
             &tags,
             self.excludes(Format::Wheel)?,
+            self.compression_level,
         )?;
         self.add_external_libs(&mut writer, &[&artifact], &[ext_libs])?;
 
@@ -900,6 +944,7 @@ impl BuildContext {
             &self.metadata24,
             &tags,
             self.excludes(Format::Wheel)?,
+            self.compression_level,
         )?;
         self.add_external_libs(&mut writer, &[&artifact], &[ext_libs])?;
 
@@ -998,6 +1043,7 @@ impl BuildContext {
             &metadata24,
             &tags,
             self.excludes(Format::Wheel)?,
+            self.compression_level,
         )?;
 
         if self.project_layout.python_module.is_some() && self.target.is_wasi() {
@@ -1231,7 +1277,7 @@ fn emscripten_version() -> Result<String> {
 fn emcc_version() -> Result<String> {
     use std::process::Command;
 
-    let emcc = Command::new("emcc")
+    let emcc = Command::new(if cfg!(windows) { "emcc.bat" } else { "emcc" })
         .arg("-dumpversion")
         .output()
         .context("Failed to run emcc to get the version")?;
