@@ -23,7 +23,7 @@ pub const MINIMUM_PYTHON_MINOR: usize = 7;
 pub const MINIMUM_PYPY_MINOR: usize = 8;
 /// Be liberal here to include preview versions
 pub const MAXIMUM_PYTHON_MINOR: usize = 13;
-pub const MAXIMUM_PYPY_MINOR: usize = 10;
+pub const MAXIMUM_PYPY_MINOR: usize = 11;
 
 /// Identifies conditions where we do not want to build wheels
 fn windows_interpreter_no_build(
@@ -96,9 +96,10 @@ fn windows_interpreter_no_build(
 /// platforms.
 fn find_all_windows(
     target: &Target,
-    min_python_minor: usize,
+    bridge: &BridgeModel,
     requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<String>> {
+    let min_python_minor = bridge.minimal_python_minor_version();
     let code = "import sys; print(sys.executable or '')";
     let mut interpreter = vec![];
     let mut versions_found = HashSet::new();
@@ -218,7 +219,7 @@ fn find_all_windows(
     }
 
     // Fallback to pythonX.Y for Microsoft Store versions
-    for minor in min_python_minor..=MAXIMUM_PYTHON_MINOR {
+    for minor in min_python_minor..=bridge.maximum_python_minor_version() {
         if !versions_found.contains(&(3, minor)) {
             let executable = format!("python3.{minor}.exe");
             if let Some(python_info) = windows_python_info(Path::new(&executable))? {
@@ -732,12 +733,11 @@ impl PythonInterpreter {
         requires_python: Option<&VersionSpecifiers>,
         bridge: Option<&BridgeModel>,
     ) -> Vec<PythonInterpreter> {
-        let bindings = bridge.and_then(|bridge| bridge.bindings());
-        let min_python_minor = bindings
-            .map(|bindings| bindings.minimal_python_minor_version())
+        let min_python_minor = bridge
+            .map(|bridge| bridge.minimal_python_minor_version())
             .unwrap_or(MINIMUM_PYTHON_MINOR);
-        let min_pypy_minor = bindings
-            .map(|bindings| bindings.minimal_pypy_minor_version())
+        let min_pypy_minor = bridge
+            .map(|bridge| bridge.minimal_pypy_minor_version())
             .unwrap_or(MINIMUM_PYPY_MINOR);
         let supports_free_threaded = bridge
             .map(|bridge| bridge.supports_free_threaded())
@@ -794,23 +794,12 @@ impl PythonInterpreter {
         bridge: &BridgeModel,
         requires_python: Option<&VersionSpecifiers>,
     ) -> Result<Vec<PythonInterpreter>> {
-        let min_python_minor = match bridge {
-            BridgeModel::Bindings(bindings) | BridgeModel::Bin(Some(bindings)) => {
-                bindings.minimal_python_minor_version()
-            }
-            _ => MINIMUM_PYTHON_MINOR,
-        };
-        let min_pypy_minor = match bridge {
-            BridgeModel::Bindings(bindings) | BridgeModel::Bin(Some(bindings)) => {
-                bindings.minimal_pypy_minor_version()
-            }
-            _ => MINIMUM_PYPY_MINOR,
-        };
         let executables = if target.is_windows() {
             // TOFIX: add PyPy support to Windows
-            find_all_windows(target, min_python_minor, requires_python)?
+            find_all_windows(target, bridge, requires_python)?
         } else {
-            let mut executables: Vec<String> = (min_python_minor..=MAXIMUM_PYTHON_MINOR)
+            let mut executables: Vec<String> = (bridge.minimal_python_minor_version()
+                ..=bridge.maximum_python_minor_version())
                 .filter(|minor| {
                     requires_python
                         .map(|requires_python| {
@@ -821,12 +810,9 @@ impl PythonInterpreter {
                 .map(|minor| format!("python3.{minor}"))
                 .collect();
             // Also try to find PyPy for cffi and pyo3 bindings
-            if *bridge == BridgeModel::Cffi
-                || bridge.is_bindings("pyo3")
-                || bridge.is_bindings("pyo3-ffi")
-            {
+            if *bridge == BridgeModel::Cffi || bridge.is_pyo3() {
                 executables.extend(
-                    (min_pypy_minor..=MAXIMUM_PYPY_MINOR)
+                    (bridge.minimal_pypy_minor_version()..=bridge.maximum_pypy_minor_version())
                         .filter(|minor| {
                             requires_python
                                 .map(|requires_python| {
@@ -1035,15 +1021,14 @@ fn calculate_abi_tag(ext_suffix: &str) -> Option<String> {
 
 #[cfg(test)]
 mod tests {
-    use crate::Bindings;
+    use crate::bridge::{PyO3, PyO3Crate};
     use expect_test::expect;
 
     use super::*;
 
     #[test]
     fn test_find_interpreter_by_target() {
-        let target =
-            Target::from_target_triple(Some("x86_64-unknown-linux-gnu".to_string())).unwrap();
+        let target = Target::from_resolved_target_triple("x86_64-unknown-linux-gnu").unwrap();
         let pythons = PythonInterpreter::find_by_target(&target, None, None)
             .iter()
             .map(ToString::to_string)
@@ -1060,6 +1045,7 @@ mod tests {
                 "PyPy 3.8",
                 "PyPy 3.9",
                 "PyPy 3.10",
+                "PyPy 3.11",
             ]
         "#]];
         expected.assert_debug_eq(&pythons);
@@ -1068,9 +1054,11 @@ mod tests {
         let pythons = PythonInterpreter::find_by_target(
             &target,
             None,
-            Some(&BridgeModel::Bindings(Bindings {
-                name: "pyo3".to_string(),
+            Some(&BridgeModel::PyO3(PyO3 {
+                crate_name: PyO3Crate::PyO3,
                 version: semver::Version::new(0, 23, 0),
+                abi3: None,
+                metadata: None,
             })),
         )
         .iter()
@@ -1088,6 +1076,7 @@ mod tests {
                 "CPython 3.13t",
                 "PyPy 3.9",
                 "PyPy 3.10",
+                "PyPy 3.11",
             ]
         "#]];
         expected.assert_debug_eq(&pythons);
@@ -1111,6 +1100,7 @@ mod tests {
                 "PyPy 3.8",
                 "PyPy 3.9",
                 "PyPy 3.10",
+                "PyPy 3.11",
             ]
         "#]];
         expected.assert_debug_eq(&pythons);
@@ -1130,6 +1120,7 @@ mod tests {
                 "CPython 3.12",
                 "CPython 3.13",
                 "PyPy 3.10",
+                "PyPy 3.11",
             ]
         "#]];
         expected.assert_debug_eq(&pythons);
@@ -1137,9 +1128,11 @@ mod tests {
         let pythons = PythonInterpreter::find_by_target(
             &target,
             Some(&VersionSpecifiers::from_str(">=3.8").unwrap()),
-            Some(&BridgeModel::Bindings(Bindings {
-                name: "pyo3".to_string(),
+            Some(&BridgeModel::PyO3(PyO3 {
+                crate_name: PyO3Crate::PyO3,
                 version: semver::Version::new(0, 23, 0),
+                abi3: None,
+                metadata: None,
             })),
         )
         .iter()
@@ -1157,6 +1150,7 @@ mod tests {
                 "CPython 3.13t",
                 "PyPy 3.9",
                 "PyPy 3.10",
+                "PyPy 3.11",
             ]
         "#]];
         expected.assert_debug_eq(&pythons);
